package com.jty.wsxt.domain.model.paper;

import com.alibaba.fastjson.JSON;
import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.module.Module;
import com.jty.wsxt.domain.shared.IdentifiedEntityObject;
import com.jty.wsxt.infrastructure.support.BusinessException;
import com.jty.wsxt.infrastructure.support.ResultCode;
import com.jty.wsxt.interfaces.dto.PaperDto;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import javax.persistence.Entity;
import java.time.LocalDateTime;

/**
 * Paper
 *
 * @author Manjiajie
 * @since 2019-6-21 10:41:39
 */
@Data
@Entity
public class Paper extends IdentifiedEntityObject<Module> {

    private String name;

    private Integer moduleId;

    private Integer creatorId;

    private String creatorName;

    private Integer subjectId;

    private Integer typeId;

    private Integer specialistId;

    private String specialistName;

    private String answer;

    private String firstFeedback;

    private String secondFeedback;

    private LocalDateTime feedbackTime;

    @Override
    public boolean sameIdentityAs(Module other) {
        return false;
    }

    public void save(){
        DomainRegistry.paperRepository().save(this);
    }

    public void delete(){
        DomainRegistry.paperRepository().delete(this);
    }

    public PaperDto cashToPaperDto(){
        PaperDto paperDto = new PaperDto();
        paperDto.setCreatorName(this.creatorName);
        paperDto.setModuleName(DomainRegistry.moduleRepository().findById(this.moduleId).orElseThrow(()-> new BusinessException(ResultCode.DATA_IS_WRONG)).getName());
        paperDto.setTypeName(DomainRegistry.typeRepository().findById(this.typeId).orElseThrow(()-> new BusinessException(ResultCode.DATA_IS_WRONG)).getName());
        paperDto.setSpecialistName(this.specialistName);
        paperDto.setName(this.name);
        if (StringUtils.isNotEmpty(this.answer)) {
            paperDto.setAnswer(JSON.parseArray(this.answer, String.class));
        }
        if (StringUtils.isNotEmpty(this.firstFeedback)) {
            paperDto.setFirstFeedback(JSON.parseArray(this.firstFeedback, String.class));
        }
        if (StringUtils.isNotEmpty(this.secondFeedback)) {
            paperDto.setSecondFeedback(JSON.parseArray(this.secondFeedback, String.class));
        }
        paperDto.setFeedbackTime(this.feedbackTime);
        return paperDto;
    }
}
